#include "gtkimage.h"
#include <highgui.h>

extern GtkWidget *window_correlation;
extern GtkWidget *Application;
extern motion_t motion;

#include "geometry.h"
static pt2d_t inverse_transform(projection_t *data,
                                pt2d_t from)
{
    pt2d_t to;
    pt3d_t xx, yy, cm;

    xx = multiply(from.x, data->vx);
    yy = multiply(from.y, data->vy);

    cm = add(data->Os, add(xx, yy));

    to.x = cm.x * data->d / cm.z;
    to.y = cm.y * data->d / cm.z;

    return to;
}

static int getpixel_fast_aa(image_t *img, projection_t *data,
                            double x, double y, int offsetx,
                            int offsety, int *r, int *g, int *b)
{
    double i0,j0,i1,j1,I_,J_;
    int pR,pB,pG;
    double c1,c2,c3,c4;
    pt2d_t from,to;

    from.x = x;
    from.y = y;

    /*Point normé => point dénormé*/
    to = inverse_transform(data, from);

    /* Pourcentage de couleur */
    j0 = to.x;
    i0 = to.y;
    j1 = to.x + 1.0;
    i1 = to.y + 1.0;

    J_ = (int)to.x;
    I_ = (int)to.y;

    c1 = ((I_+1) - i0) * ((J_+1) - j0);
    c2 = ((I_+1) - i0) * (j1 - (J_+1));
    c3 = (i1 - (I_+1)) * (j1 - (J_+1));
    c4 = (i1 - (I_+1)) * ((J_+1) - j0);
    pR = pB = pG = 0;
    getpixel_fast(img,(int)to.x+offsetx,(int)to.y+offsety,r,g,b);
    pR = c1 * (*r);
    pG = c1 * (*g);
    pB = c1 * (*b);
    getpixel_fast(img,(int)to.x+1+offsetx,(int)to.y+offsety,r,g,b);
    pR += c2 * (*r);
    pG += c2 * (*g);
    pB += c2 * (*b);
    getpixel_fast(img,(int)to.x+1+offsetx,(int)to.y+1+offsety,r,g,b);
    pR += c3*(*r);
    pG += c3*(*g);
    pB += c3*(*b);
    getpixel_fast(img,(int)to.x+offsetx,(int)to.y+1+offsety,r,g,b);
    pR += c4*(*r);
    pG += c4*(*g);
    pB += c4*(*b);

    *r = pR;
    *g = pG;
    *b = pB;

    return 0;
}


/*Chargement des images*/
int gtkLoadImage(char **file,pair_t *pair,double factor)
{
	int i;
	//image_t tmp;
	/*Lecture des fichiers*/
	for(i=0; (i<2)&&file[i];i++)
//	for(i=1; ;i--)
	{
		printf("Chargement de %s\n",file[i]);
		/*Chargement*/
		//if(image_load(&tmp,file[i])==-1)
		if(image_load(&pair->image[i],file[i])==-1)
			return -1;		
		else
		{
		   //image_new(&(pair->image[i]),tmp.width*factor,tmp.height*factor,3);
		   //image_resize(&tmp,&(pair->image[i]),factor);
		   image_resize(&pair->image[i],factor);
		}
	}
	return i;
//	return 1;
}
int gtkReleaseImage(GtkWidget *widget)
{
    image_t *image = g_object_get_data((GObject *)widget,"image");
   
    image_free(image);
    return 0;
}
/*
 * Création des PixBuf
 */
GdkPixbuf* image_to_pix(image_t *img)
{
    int width,height;
    GdkPixbuf *pix;
    /*Copie de l'image */
    pix = gdk_pixbuf_new_from_data(img->pixels,
   				   GDK_COLORSPACE_RGB,
   				   FALSE,
   				   8,
				   img->width,
				   img->height,
   				   img->pitch,
				   NULL,
    				   NULL);
    width =gdk_pixbuf_get_width(pix) ;
    height =gdk_pixbuf_get_height(pix) ;
    /*Redimensonnement de l'image*/
//    pix = gdk_pixbuf_scale_simple(pix,
//                                 400,400*((double)height/(double)width),
//				 GDK_INTERP_NEAREST);
    				 
    width = gdk_pixbuf_get_width(pix);
    height = gdk_pixbuf_get_height(pix);
    return pix;
}
GdkColor Color(int r,int g,int b)
{
	GdkColor color;
	color.red = 65535*r;
    	color.green = 65535*g;
    	color.blue = 65535*b;
    	return color;
}
int GetHeight(GtkWidget* Area)
{
	return ((GtkDrawingArea*)Area)->widget.requisition.height;
}
int GetWidth(GtkWidget* Area)
{
	return ((GtkDrawingArea*)Area)->widget.requisition.width;
}
static GtkFixedChild*
get_child (GtkFixed  *fixed,
           GtkWidget *widget)
{
  GList *children;
  
  children = fixed->children;
  while (children)
    {
      GtkFixedChild *child;
      
      child = children->data;
      children = children->next;

      if (child->widget == widget)
        return child;
    }

  return NULL;
}
int gtkDisplay(GtkWidget *Area)
{
    image_t *img,*img_rgb,*tmp;
    int width,height;
    GdkPixbuf *pix;
    GtkWidget *wid;
    float factor;
    gchar *text;
    GtkFixedChild *child;
    printf("Display : %s, %p\n",Area->name,Area);
//    return 0;
    img = g_object_get_data ((GObject *)Area, "image");      
//    printf("gtkDisplay -> image : %p\n",img);
//    printf("gtkDisplay -> img na: %s\n",Area->name);

    if(!img)
    {
	return -1;
    }
    
    /*Conversion BGR => RGB*/
    img_rgb = malloc(sizeof(image_t));
    image_new(img_rgb,img->width,img->height,3);
    convert_BGR2RGB(img,img_rgb);
    
    if(strstr(Area->name,"drawing") != NULL)
        wid  = lookup_widget((GtkWidget*)Application,"spinbutton_images");
    else
        wid  = lookup_widget((GtkWidget*)Application,"spinbutton_correlation");
    text = gtk_editable_get_chars((GtkEditable*)wid,0,-1);
    factor = atof(text);
    
    tmp = malloc(sizeof(image_t));
    image_new(tmp,img_rgb->width*factor,img_rgb->height*factor,3);
//    image_resize(img_rgb,tmp,factor);
    image_resize(img_rgb,factor);
    /*Conversion image_t => pixbuf*/
    pix = image_to_pix(img_rgb);
//    pix = image_to_pix(tmp);
    /*Affichage de l'image*/
    width = gdk_pixbuf_get_width(pix);
    height = gdk_pixbuf_get_height(pix);

    
//    width = img->width;
//    height = img->height;
    
    gdk_pixbuf_render_to_drawable(pix,
				Area->window ,      //GdkDrawable*drawable,
				Area->style->white_gc,//GdkGC *gc,
				0,	  	      //src_x,
				0,	              //src_y,
				0,			//dest_x,
				0,			//dest_y,
				width,	//width,
				height,	//height,
				GDK_RGB_DITHER_NONE,		//dither,
				0, 			   //x_dither,
				0);  			   //y_dither);
	printf("size : %d %d\n",width,height);
    if(GetHeight(Area) != height || GetWidth(Area) != width)
    {
        printf("resize area**************************************************************\n");
	printf("%d %d => %d %d\n",GetWidth(Area),GetHeight(Area),height,width);
        //gtk_drawing_area_size((GtkDrawingArea*)Area,width,height);		
        gtk_drawing_area_size((GtkDrawingArea*)Area,width,height);
//GtkFixedChild* get_child (Area->parent,Area);
	child = get_child ((GtkFixed*)Area->parent,Area);
	printf("child : %d %d\n",child->x,child->y);
  
	if(child->x != 0) // image de droite
            gtk_fixed_move  ((GtkFixed*)Area->parent,Area,width+10,child->y);
	   
	
//	printf("%d %d\n",((GtkFixed *)Area->parent)->children[0].x,Area->y);
//        gtk_fixed_move  (Area->parent,Area,50,50);
//        gtk_drawing_area_size((GtkDrawingArea*)Area,100,10);		
    }
    image_free(img_rgb);
    free(img_rgb);
    free(pix);
    return 0;
}

/*Affichage des points*/
int gtkPlot(GtkWidget *Area,ptsing_t *ptsing,unsigned int count,int width,int height)
{
    if(Area )
    {
        int i;
    	int w,h;
    	int diametre;
    	GdkColor	color_red,color_green,color_blue;
    	GdkGC		*GC;
	//GdkFont 	*font;
	/**/
	diametre = 5;
    	/*Configuration et génération des contextes graphique*/
    	GC = gdk_gc_new ((Area)->window);
    	color_red = Color(1,0,0);
    	color_green = Color(0,1,0);
    	color_blue = Color(0,0,1);
	/*Dimension du conteneur */   
    	w = GetWidth(Area);
    	h = GetHeight(Area);
/*	        gdk_gc_set_rgb_fg_color (GC, &color_green);
font = gdk_font_load  ("-misc-fixed-medium-r-normal--0-0-75-75-c-0-iso8859-1" );
   printf("%p \n",font); 
 gdk_draw_text      (Area->window,
			 font,
			 GC,
			 50,
			 50,
			 "coucou",
			 10);
			 */
	/*Affichage des points*/    
    	for(i=0 ; i<count ; i++)
    	{
	    if(ptsing[i].score >= 1)
	        gdk_gc_set_rgb_fg_color (GC, &color_green);
	    else
	    	gdk_gc_set_rgb_fg_color (GC, &color_red);
    	    gdk_draw_rectangle(	(Area)->window,
                              	GC,
                                FALSE,
                                (int)(ptsing[i].point.x*w/width)-(diametre/2),
                                (int)(ptsing[i].point.y*h/height)-(diametre/2),
    				diametre,
    				diametre);
    	}
	return 0;
    }
    return -1;
}
/*Affichage des points*/
int gtkLine(GtkWidget *Area,scanline_t *scanline,unsigned int count,int width,int height)
{
	if(Area )
	{
		int i;
		int x1,y1,x2,y2;
		int h,w;
    	GdkColor	color_red;
    	GdkGC		*GC;
//    	projection_t *projection;
		/*Recuperation des points*/
  //  	projection = g_object_get_data (G_OBJECT ((Area)), "myprojection");
    //	if(!projection)
    //		return -1;    	
    	GC = gdk_gc_new ((Area)->window);
    	color_red.red = 65535;
    	color_red.green = 65535*0;
    	color_red.blue = 65535*0;
    	gdk_gc_set_rgb_fg_color (GC, &color_red);
    	w = GetWidth(Area);
    	h = GetHeight(Area);
		for(i=0 ; i<count; i+=20)
		{
              x1 = scanline[i].a.x * 1;
              y1 = scanline[i].a.y * 1;
              x2 = scanline[i].b.x * 1;
              y2 = scanline[i].b.y * 1;
			gdk_draw_line ( Area->window,GC, x1*w/width,y1*h/height,x2*w/width,y2*h/height);				
		}
	}
	return 0;
}

/* Gestion des Drawing Area
 *
 */
gboolean
on_drawingarea_images_expose_event_     (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
//    static int cpt=0;
    /*Point singuliers*/
    ptsing_t *pt;
    unsigned int *pt_count;
    /*scanline*/
    scanline_t *scanline;
    unsigned int *scanline_count;
    GtkWidget *wid;
    image_t *image;
    /*Affichage de l'image*/
    gtkDisplay(widget);
    image      = g_object_get_data(G_OBJECT(widget),"image"); 
//    printf("drawing -> image : %p\n",image);
    if(!image)
     return TRUE;
    /*Récupération des structures correspondantes*/
    pt = g_object_get_data(G_OBJECT(widget),"point"); 
//    printf("drawing -> pt    : %p\n",pt);
    pt_count = (unsigned int*)g_object_get_data(G_OBJECT(widget),"point_count"); 
//    printf("drawing -> pt ct : %p\n",pt_count);
    scanline = g_object_get_data(G_OBJECT(widget),"scanline"); 
//    printf("drawing -> scanl : %p\n",scanline);
    scanline_count = (unsigned int*)g_object_get_data(G_OBJECT(widget),"scanline_count");
//    printf("drawing -> sc pt : %p\n",scanline_count);

    /*Affichage des points*/
    wid = lookup_widget((GtkWidget*)Application, "checkbutton_point");
    if(pt && gtk_toggle_button_get_active((GtkToggleButton*)wid) )
    {

        printf("Affichage des points suivi\n");
        gtkPlot(widget,
	        pt,*pt_count,
                image->width,image->height);
    }
    /*Affichage des scanlines*/
    wid = lookup_widget((GtkWidget*)Application, "checkbutton_scanline");
    if(scanline && gtk_toggle_button_get_active((GtkToggleButton*)wid) )
    {
        printf("Affichage des scanline suivi\n");
        gtkLine(widget,
	        scanline,*scanline_count,
                image->width,image->height);

    }
    
//    printf("refresh %d\n",cpt++);
    return TRUE;
}

int gtkCorrelation(current_motion_t *current_motion)
{
    GtkWidget *wid,*widget;
    pair_t *structure;
    map_t *Map;
    projection_t *projection;
    int x,ycorr,pixel,y;
    //image_t* tmp;
    static image_t *line=NULL;
    //float factor;
    //gchar *text;
    widget = current_motion->widget;
    y = current_motion->y;
    
 if(strcmp(widget->name,"drawing0")!= 0 )
      return -1;

  // printf("IN gtkCorrelation %s\n",widget->name);


//   return 0;
    structure  = (pair_t*)lookup_widget((GtkWidget*)widget, "pair");
    Map=structure->correlation[LEFT].Map;
    projection= (structure->projection);

//    printf("%d\n",y);
    
    if(!Map)
    {
        printf("map not build\n");
        return FALSE;
    }
    if(!line)
    {
        line = malloc(sizeof(image_t));
	image_new(line,
	  	  (projection[LEFT].right-projection[LEFT].left+3),
	  	  (projection[RIGHT].right-projection[RIGHT].left+3),
		  3);
        /*Récupération widget correlation*/
        wid  = lookup_widget((GtkWidget*)window_correlation, "fixed_handle");  
        /*Récupération du drawing Area*/
        wid  = gtk_object_get_data((GtkObject*)wid, "line");  
        /*Affectation de l'image*/
        gtk_object_set_data ((GtkObject *)wid, "image",line);
	printf("CREATE LINE\n");
    }
    else
    {
    }
    y = (int)(y)*Map->height/GetHeight(widget); 
    for(x=0 ; x<Map->width ; x++)
    {
        for(ycorr = 0; ycorr < TRANS_X; ycorr++)
        {
            pixel = (Map->elem+y*Map->width+x)->tabcorr[ycorr];
	    //int ypix = TRANS_X - 1 - ycorr;
	    int ypix = (line->height-1-3)-ycorr-x+TRANS_X/2;
	    //setpixel_fast(line, x+3, ypix, pixel, pixel, pixel);
	    if(ycorr == (Map->elem+y*Map->width+x)->depth + TRANS_X / 2)
	    {
	        if((Map->elem+y*Map->width+x)->highlight)
	   	    setpixel_fast(line, x+3, ypix, 255, 255, 0);
	   	    //setpixel_fast(line, xpix, ypix, 255, 255, 0);
		else if((Map->elem+y*Map->width+x)->invalid)
		    setpixel_fast(line, x+3, ypix, 0, 255, 0);
		    //setpixel_fast(line, xpix, ypix, 0, 255, 0);
		  else
		      setpixel_fast(line, x+3, ypix, 255, 0, 0);
		      //setpixel_fast(line, xpix, ypix, 255, 0, 0);
	    }
	    else if(ycorr == (Map->elem+y*Map->width+x)->depth1 + TRANS_X / 2)
	    {
	  	    setpixel_fast(line, x+3, ypix, 0, 0, 255);
	    }
	    else if(ycorr == (Map->elem+y*Map->width+x)->depth2 + TRANS_X / 2)
	    {
	  	    setpixel_fast(line, x+3, ypix, 0, 0, 255);
	    }
	    else
	    {
	      //pixel *= 2;
	      pixel /= 128;
	      if(pixel > 255)
		  //setpixel_fast(line, xpix, ypix, 255, 255, 255);
		  //setpixel_fast(line, x+3, ypix, 0, 0, 255);
		  //setpixel_fast(line, x+3, ypix, 255, 255, 255);
		  setpixel_fast(line, x+3, ypix, 128, 128, 0);
		  //setpixel_fast(line, xpix, ypix, 0, 0, 255);
	      else
		  setpixel_fast(line, x+3, ypix, pixel,pixel,pixel);
		  //setpixel_fast(line, xpix, ypix, pixel,pixel,pixel);
	    }
	}
    }
    for(x=0 ; x<line->width ; x++)
    {
        int r,g,b;
        for(ycorr=0 ; ycorr<3 ; ycorr++)
	{
    	     /*Récupération des trois lignes de l'image gauche*/
	     getpixel_fast_aa(&structure->image[LEFT],
	       		      &projection[LEFT],
                             (int)projection[LEFT].left + x,
                             (int)projection[LEFT].top + y+ycorr-1 + structure->adjust,
                             0, 0,
                             &r,
                             &g,
                             &b);
	     /*Affichage horizontal en bas*/
	     setpixel_fast(line,x+3,ycorr+line->height-3,r,g,b);
	}
    }
    for(x=0 ; x<line->height ; x++)
    {
        int r,g,b;
        for(ycorr=0 ; ycorr<3 ; ycorr++)
	{
    	     /*Récupération des trois lignes de l'image droite */
	     getpixel_fast_aa(&structure->image[RIGHT],
	       		      &projection[RIGHT],
                             (int)projection[RIGHT].left + x,
                             (int)projection[RIGHT].top + y+(3-ycorr-1),
                             0, 0,
                             &r,
                             &g,
                             &b);
	     /*Affichage vertical à gauche*/
	     setpixel_fast(line,ycorr,(line->height-1)-3-x,r,g,b);
	}
    }
/*
   tmp = malloc(sizeof(image_t));
   wid  = lookup_widget((GtkWidget*)Application,"spinbutton_correlation");
   text = gtk_editable_get_chars((GtkEditable*)wid,0,-1);
   factor = atof(text);
#if 1
   image_new(tmp,line->width*factor,line->height*factor,3);
   image_resize(line,tmp,factor);
#else
   image_new(tmp,line->width,line->height,3);
    image_copy(line,tmp);
#endif
*/    
//   tmp = malloc(sizeof(image_t));
//   image_new(tmp,line->width,line->height,3);
//    image_copy(line,tmp);
   
//    image_resize(line,);
    
    /*Récupération widget correlation*/
    wid  = lookup_widget((GtkWidget*)window_correlation, "fixed_handle");  
    
   // printf("parent line : %s\n",wid->name);
    /*Récupération du drawing Area*/
    wid  = gtk_object_get_data((GtkObject*)wid, "line");  
//    gtk_drawing_area_size((GtkDrawingArea*)wid,line->width*factor,line->height*factor);
    /*Affectation de l'image*/
    gtk_object_remove_data ((GtkObject *)wid, "image");
    //gtk_object_set_data ((GtkObject *)wid, "image",tmp);
    gtk_object_set_data ((GtkObject *)wid, "image",line);
    /*Redraw de line*/
    gtk_widget_queue_draw(wid);
   
    image_save("1line.bmp",line);
   return 0;
}
gboolean
on_drawingarea_images_clic_event_     (GtkWidget       *widget,
                                         GdkEventButton *event,
							              gpointer         user_data)
{
/*    GtkWidget *wid;
    pair_t *structure;
    map_t *Map;
    projection_t *projection;
    int x,ycorr,pixel,y;
    static image_t *line=NULL;
  */  

 
    current_motion.widget = widget;
    current_motion.y = event->y; 
    motion = asked;


    
    //current_motion.y = (int)(event->y)*Map->height/GetHeight(widget); 
//    if(motion==asked)
//       printf("motion already asked\n"); 
//    else
//       printf("motion asked\n"); 
    //printf("motion asked\n"); 
   // gtkCorrelation(widget,(event->y)*Map->height/GetHeight(widget);
#if 0
    
 //   if(!(strstr(widget->name,"drawing")== NULL && motion == 1))
//     return ;

  //  motion = 0;
    
    if(strcmp(widget->name,"drawing0")!= 0 )
      return -1;
    structure  = (pair_t*)lookup_widget((GtkWidget*)widget, "pair");
    Map=structure->correlation[LEFT].Map;
    projection= (structure->projection);
    //wid = gtk_object_get_data ((GtkObject *)widget, "drawing0");

    if(!Map)
    {
        printf("map not build\n");
        return FALSE;
    }
//    printf("clic sur %s (%g %g)\n",widget->name,event->x,event->y);	
//    printf("%g = %d\n",event->x*Map->width/GetWidth(widget),Map->width);
//    printf("%g = %d\n",event->y*Map->height/GetHeight(widget),Map->height);
//    fflush(stdout);
    if(!line)
    {
        line = malloc(sizeof(image_t));
        //image_new(line,Map->width,TRANS_X,3);
//        image_new(line,Map->width,Map->height,3);
	image_new(line,
	  	  projection[LEFT].right-projection[LEFT].left+3,
	  	  projection[RIGHT].right-projection[RIGHT].left+3,
		  3);
		  
    }
    else
    {
 /*       if(line->width != Map->width)
	{
            image_free(line);
	    free(line);
            line = malloc(sizeof(image_t));
            image_new(line,Map->width,TRANS_X,3);
	}
	*/
    }
    
    y = (event->y)*Map->height/GetHeight(widget);
   

    for(x=0 ; x<line->width ; x++)
    {
        int r,g,b;
        for(ycorr=0 ; ycorr<3 ; ycorr++)
	{
    	     /*Récupération des trois lignes de l'image gauche*/
	     getpixel_fast_aa(&structure->image[LEFT],
	       		      &projection[LEFT],
                             (int)projection[LEFT].left + x,
                             (int)projection[LEFT].top + y+ycorr-1 + structure->adjust,
                             0, 0,
                             &r,
                             &g,
                             &b);
	     /*Affichage horizontal en bas*/
	     setpixel_fast(line,x+3,ycorr+line->height-3,r,g,b);
	}
    }
    for(x=0 ; x<line->height ; x++)
    {
        int r,g,b;
        for(ycorr=0 ; ycorr<3 ; ycorr++)
	{
    	     /*Récupération des trois lignes de l'image droite */
	     getpixel_fast_aa(&structure->image[RIGHT],
	       		      &projection[RIGHT],
                             (int)projection[RIGHT].left + x,
                             (int)projection[RIGHT].top + y+(3-ycorr-1),
                             0, 0,
                             &r,
                             &g,
                             &b);
	     /*Affichage vertical à gauche*/
	     setpixel_fast(line,ycorr,(line->height-1)-3-x,r,g,b);
	}
    }
    for(x=0 ; x<Map->width ; x++)
    {
        for(ycorr = 0; ycorr < TRANS_X; ycorr++)
        {
            pixel = (Map->elem+y*Map->width+x)->tabcorr[ycorr];
	    //int ypix = TRANS_X - 1 - ycorr;
	    int ypix = (line->height-1-3)-ycorr-x;

	    
	    if(ycorr == (Map->elem+y*Map->width+x)->depth + TRANS_X / 2)
	    {
	        if((Map->elem+y*Map->width+x)->highlight)
	   	    setpixel_fast(line, x+3, ypix, 255, 255, 0);
	   	    //setpixel_fast(line, xpix, ypix, 255, 255, 0);
		else if((Map->elem+y*Map->width+x)->invalid)
		    setpixel_fast(line, x+3, ypix, 0, 255, 0);
		    //setpixel_fast(line, xpix, ypix, 0, 255, 0);
		  else
		      setpixel_fast(line, x+3, ypix, 255, 0, 0);
		      //setpixel_fast(line, xpix, ypix, 255, 0, 0);
	    }
	    else
	    {
	      //pixel *= 2;
	      pixel /= 128;
	      if(pixel > 255)
		  //setpixel_fast(line, xpix, ypix, 255, 255, 255);
		  setpixel_fast(line, x+3, ypix, 0, 0, 255);
		  //setpixel_fast(line, xpix, ypix, 0, 0, 255);
	      else
		  setpixel_fast(line, x+3, ypix, pixel,pixel,pixel);
		  //setpixel_fast(line, xpix, ypix, pixel,pixel,pixel);
	    }
	}
    }

    //wid  = lookup_widget((GtkWidget*)widget, "fixed_images");  
    wid  = lookup_widget((GtkWidget*)window_correlation, "fixed_handle");  
    printf("parent line : %s\n",wid->name);
    wid  = gtk_object_get_data((GtkObject*)wid, "line");  
    
    gtk_object_set_data ((GtkObject *)wid, "image",line);

//    wid  = lookup_widget((GtkWidget*)widget, "fixed_images");  
    gtk_widget_queue_draw(wid);
    image_save("1line.bmp",line);
//    image_display(line);
//    cvWaitKey(0);
#endif
#if 0
     for(y=0 ; y<left_depthmap_norm->height ; y++)
        {
            for(x=0 ; x<left_depthmap_norm->width ; x++)
            {
                for(ycorr = 0; ycorr < TRANS_X; ycorr++)
                {
                   int xpix = x, ypix = y * TRANS_X + (TRANS_X - 1 - ycorr);
                   pixel = (Map->elem+y*Map->width+x)->tabcorr[ycorr];
                   if(ycorr == (Map->elem+y*Map->width+x)->depth + TRANS_X / 2)
                   {
                       if((Map->elem+y*Map->width+x)->highlight)
                           setpixel_fast(line, xpix, ypix, 255, 255, 0);
                       else if((Map->elem+y*Map->width+x)->invalid)
                           setpixel_fast(line, xpix, ypix, 0, 255, 0);
                       else
                           setpixel_fast(line, xpix, ypix, 255, 0, 0);
                   }
                   else
                   {
                       //pixel *= 2;
                       pixel /= 128;
                       if(pixel > 255)
                           //setpixel_fast(line, xpix, ypix, 255, 255, 255);
                           setpixel_fast(line, xpix, ypix, 0, 0, 255);
                       else
                           setpixel_fast(line, xpix, ypix, pixel,pixel,pixel);
                   }
                }
            }
        }
#endif
    return TRUE;                                        
}

gboolean
on_drawingarea_motion_notify_event_    (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
   printf("*");
   fflush(stdout);
  return FALSE;
}

int CreateDrawing(GtkWidget *parent,
  		  GtkWidget *drawing,
		  gchar *name,
		  int x,int y,int width,int height)
{
	printf("Create drawing area (%s,%p)\n",name,drawing);
	gtk_widget_set_name (drawing, name);
         gtk_drawing_area_size((GtkDrawingArea*)drawing,width,height);
	gtk_widget_show(drawing);
        gtk_widget_set_events (drawing,
	  GDK_POINTER_MOTION_MASK|
	  GDK_BUTTON_PRESS_MASK);
	printf("%s (parent = %s,%p)\n",drawing->name,parent->name,parent);
	gtk_fixed_put (GTK_FIXED (parent), drawing, x,y);
	//gtk_widget_set_size_request((GtkDrawingArea*)drawing,width,height);
         gtk_drawing_area_size((GtkDrawingArea*)drawing,width,height);


	g_signal_connect ((gpointer)(drawing), "expose_event",
                    G_CALLBACK (on_drawingarea_images_expose_event_),
                    NULL);
	/*@@@@ pas d'effet de la callbacks..... */
 	    g_signal_connect ((gpointer) (drawing), "button_press_event",
                        G_CALLBACK (on_drawingarea_images_clic_event_),
                        NULL );
	    g_signal_connect ((gpointer) drawing, "motion_notify_event",
                        G_CALLBACK (on_drawingarea_images_clic_event_),
                        NULL);
	
	return 0;
}


